***********************************************************
* $VER: StartUp_S 4.00
*
* This is my TakeSystem and RestoreSystem routines that I now use
* in all my demos.  I am spreading them in hopes to get rid of the
* crapy coding practices that people have had since the C64 days.
* I'd like to thank Comrade J for his excellent "howtocode" files.
* A lot of the stuff here is based on his words of wisdom. :^).  Anyway,
* these routines, as of version 4.00, handle the following:
*		Getting the VBR.
*		Saving DMACON.
*		Saving Intena.
*		Saving and reseting WB view modes.
*		Owning/Disowning the blitter.
*		Pal/Ntsc/AGA modes.
*		Killing multitasking.
*		Killing interrupts.
*		Fixed equates for case sensitive assemblers.
*		PC relative.
*		Fully linkable code.
*		Added more conditional assembly for un-needed code.
*
* Feel free to cut and paste to fit your code (i.e. put the data and bss in
* your data and bss sections.), but please leave the CODE intact!
* If you have any questions/comments/suggestions, PLEASE contact me at
* either:
*		idr@rigel.cs.pdx.edu		(internet)
*	- or -
*		Epsilon
*		P.O.B.	1914
*		Beaverton, OR  97075-1914
*		U.S.A.
*
* Many of you may not like the fact that I use includes and system calls,
* I know I don't :), but it is the ONLY SAFE WAY TO TAKE THE SYSTEM!  To
* insure that your demo will work on any system, it's what you have to do.
*
***********************************************************
* Includes
***********************************************************

		incdir	"Devel:include/"
		include	"exec/exec_lib.i"
		include	"exec/execbase.i"
		include	"graphics/gfxbase.i"
		include	"graphics/graphics_lib.i"
		include	"hardware/custom.i"

***********************************************************
* Other Macros
***********************************************************

CALL		MACRO
		jsr	_LVO\1(a6)
		ENDM

***********************************************************
* Assembly Flags
***********************************************************

; These items can be optional...that is, they may not apply to your
; code.
CHECK_FOR_PAL	SET	1		; set to 1 if you need to know if PAL
; If you do not intend to use sprites in your code, make this 0.
CHECK_FOR_AGA	SET	1		; set to 1 if you need to know if AGA
; If you (and your mod player!) will not use interrupts, set this to 0.
CHECK_FOR_VBR	SET	1		; set to 1 if you need to know what
					;   the VBR is.
MAKE_LINKABLE	SET	1		; if you want linkable code.

***********************************************************
* Other Structs
***********************************************************

		rsreset
gfx_base	rs.l	1		; pointer to graphics base
OldView		rs.l    1		; old Work Bench view addr.
OldDMACon:	rs.w	1		; old dmacon bits
OldINTEna:	rs.w	1		; old intena bits
VectorBase:	rs.l	1		; pointer to the Vector Base
ntsc:		rs.b	1		; 0 = pal, 1 = ntsc
AGA:		rs.b	1		; 0 = ESC/OCS, 1 = AGA
SYS_SIZE	rs.b	0		; size of the struct

***********************************************************
* Other Equates
***********************************************************

; This is from the 3.0 includes which almost nobody has.
 IFND	GFXB_AA_ALICE
GFXB_AA_ALICE	SET	2
 ENDC

; These are from the 2.0 includes which many people still don't have.

 IFND	gb_ChipRevBits0
gb_ChipRevBits0	SET	$ec
 ENDC

 IFND	PALn
PALn		SET	2
 ENDC

***********************************************************

 IFNE	MAKE_LINKABLE
		xdef	TakeSystem	; call to shut system down
		xdef	RestoreSystem	; call to bring system back
 ENDC

		section TheCode,code

***********************************************************
* Inputs:  a4 : points to SystemSave
* Outputs: SystemSave is filled with correct data
*          a5 : $dff002
*          d0 : zero if success, or -1 if failure

TakeSystem:	movea.l	4.w,a6		; exec base
		lea	$dff002,a5	; custom chip base + 2

		lea	GraphicsName(pc),a1	; "graphics.library"
		moveq	#33,d0		; any version that is at least 1.2 :)
		CALL	OpenLibrary	; open it.
		move.l	d0,gfx_base(a4)	; save pointer to gfx base
		beq.w	.erexit		; if we got a NULL, then exit
		movea.l	d0,a6		; for later callls...

		move.l  gb_ActiView(a6),OldView(a4)	; save old view

		move.w	#0,a1		; clears full long-word
		CALL	LoadView	; Open a NULL view (resets display
					;   on any Amiga)

		CALL	WaitTOF		; Wait twice so that an interlace
		CALL	WaitTOF		;   display can reset.

		CALL	OwnBlitter	; take over the blitter and...
		CALL	WaitBlit	;   wait for it to finish so we
					;   safely use it as we please.

		movea.l	4.w,a6		; exec base
		CALL	Forbid		; kill multitasking

 IFNE	CHECK_FOR_PAL
		cmpi.l	#37,LIB_VERSION(a6)	; 2.04 or better?
		bge.b	.os20		; yup
		cmpi.b	#50,VBlankFrequency(a6)	; is vblank rate pal?
		beq.b	.pal		; yup.
		st	ntsc(a4)	; set NTSC flag.
		bra.b	.pal

.os20:		movea.l	gfx_base(a4),a0	; graphics base
		btst.b	#PALn,gb_DisplayFlags(a0)
		bne.b	.pal		; PAL display mode?
		st	ntsc(a4)	; set NTSC flag.
.pal:
 ENDC

 IFNE	CHECK_FOR_VBR
		moveq	#0,d0			; clear
		btst.b	#AFB_68010,AttnFlags+1(a6); are we at least a 68010?
		beq.b	.just_68000		; nope.
		move.l	a5,-(sp)		; save it.
		lea.l	vbr_exception(pc),a5	; addr of function to get VBR
		CALL	Supervisor		; supervisor state
		move.l	(sp)+,a5		; restore it.
.just_68000:	move.l	d0,VectorBase(a4)	; save it for later.
 ENDC

 IFNE	CHECK_FOR_AGA
; Take a look at HowToCode 5 or 6 to see why this is the correct
; way to take care of this.
		movea.l	gfx_base(a4),a0
		btst.b	#GFXB_AA_ALICE,gb_ChipRevBits0(a0)
		beq.b	.not_aga	; nope.
		st	AGA(a4)		; set the AGA flag.
		moveq	#0,d0
		move.w	d0,$1fc-2(a5)	; reset AGA sprites to normal mode
		move.w	d0,$106-2(a5)	; reset AGA sprites to normal mode
.not_aga:
 ENDC

		move.w	dmaconr-2(a5),d0	; old DMACON bits
		ori.w	#$8000,d0	; or it set bit for restore
		move.w	d0,OldDMACon(a4); save it

		move.w	intenar-2(a5),d0	; old INTEna bits
		ori.w	#$c000,d0	; or it set bit for restore
		move.w	d0,OldINTEna(a4); save it

		move.l	#$7fff7fff,intena-2(a5)	; kill all ints
		move.w	#$7fff,dmacon-2(a5)	; kill all dma
		moveq	#0,d0		; return no error code
		rts

.erexit:	moveq	#-1,d0		; error, don't run demo.
		rts

***********************************************************

 IFNE	CHECK_FOR_VBR
vbr_exception:
	; movec vbr,Xn is a priv. instr.  You must be supervisor to execute!
		movec   vbr,d0
	; many assemblers don't know the VBR, if yours doesn't, then use this
	; line instead.
;		dc.l	$4e7a0801
		rte				; back to user state code
 ENDC
 
***********************************************************
* Inputs:  a4 : points to SystemSave
* Outputs: d0 : zero (return code)

RestoreSystem:	lea	$dff000+dmacon,a5	; custom chip base + dmacon

	; You must do these in this order or you're asking for trouble!
		move.l	#$7fff7fff,intena-dmacon(a5)	; kill all ints
		move.w	#$7fff,(a5)			; kill all dma
		move.w	OldDMACon(a4),(a5)		; restore old dma bits
		move.w	OldINTEna(a4),intena-dmacon(a5)	; restore old int bits

		movea.l	OldView(a4),a1	; old Work Bench view
		movea.l	gfx_base(a4),a6	; gfx base
		CALL	LoadView	; Restore the view
		CALL	DisownBlitter	; give blitter back to the system.

		move.l	gb_copinit(a6),$80-dmacon(a5) ; restore system clist
		movea.l	a6,a1
		movea.l	4.w,a6		; exec base
		CALL	CloseLibrary

	; there is no call to Permit() because it is implied by the return
	; to AmigaDOS! :^)
		moveq	#0,d0
		rts

***********************************************************

; Hey!  This is a constant string!  Why put it in a different segment?
; With protected mem or re-entrant code this won't cause any problems
; at all.  Trust me. :) Besides, it makes the code smaller.

GraphicsName:	GRAFNAME		; name of gfx library
		EVEN

		END
